# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
import time, warnings
from ctypes import *
import win32con

#----------------------------------------------------------------------------------------
#def display(image0,rel_range=[200, 1000],abs_range=[], title=''):
#
#--- purpose ---
#to display one FITS image
#
#--- input parameters ---
#image0 = 2D numpy array containing data of a single image
#		images[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#rel_range = if specified, two-element list [a,b]
#		then the lower and upper thresholds used to represent the image on 256 levels are median(image0)-a and median(image0)+b
#abs_range = if specified, two-element list [a,b]
#		then the lower and upper thresholds used to represent the image on 256 levels are a and b
#		remarks:
#		- if both rel_range and abs_range are specified, rel_range is used
#title = if specified, defines the title of the window where the image is displayed.
#		This title can be used to identify the window and modify it with the functions available in win32api.py (resize, move, ...)
#		
#--- output parameters ---
#fig = matplotlib figure instance, can be used by astro.select(fig, nb_stars) function
#----------------------------------------------------------------------------------------

def display(image0,rel_range=[200., 1000.],abs_range=[], title=''):

	if len(rel_range)==2:
		dmin, dmax = rel_range
		min = np.ma.median(image0.flatten()) - dmin
		max = np.ma.median(image0.flatten()) + dmax
	elif len(abs_range)==2:
		dmin, dmax = abs_range
		min = dmin
		max = dmax
	
	image0 = (image0<min)*min + (image0>max)*max + (image0>=min)*(image0<=max)*image0
	plt.ion()
	fig = plt.figure() #facecolor='w')
	h = plt.imshow(image0, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = min, vmax =max, aspect='equal', extent=None)
	plt.colorbar()
	if title!='':
		fig.canvas.set_window_title(title) 
	plt.draw()
	return fig

#----------------------------------------------------------------------------------------
#def select(fig,nb_stars):
#
#--- purpose ---
#to select an arbitrary number of (x,y) positions on a displayed image
#usually this function is used to select stars
#selections are made with a mouse right-click, so that the left click can still be used to zoom in and out in the image
#
#--- input parameters ---
#fig = matplotlib figure instance, as returned by astro.displaying(image_data)
#nb_stars = number of (x,y) positions to be selected
#		
#--- output parameters ---
#xy = list of 2-elements lists, in the form: [[x1,y1],...,[xn,yn]]
#		each 2-element list gives the [x,y] coordinates of a selected position 
#----------------------------------------------------------------------------------------

def select(fig,nb_stars):

	xy = []
	cid = 0
	warnings.simplefilter("ignore")

	#------ here same code as for astro.display(), with plt.draw() replaced by plt.show() + connect statement ------
	def on_click(event):
		if event.inaxes is not None:
			if event.button==3:
				if len(xy)!=nb_stars:
					xy.append([int(round(event.ydata)),int(round(event.xdata))])
					print 'x=',xy[-1][0],'y=',xy[-1][1]
					if len(xy)==nb_stars:
						fig.canvas.mpl_disconnect(cid)
						fig.canvas.stop_event_loop_default()

	cid = fig.canvas.mpl_connect('button_press_event', on_click)
	fig.canvas.start_event_loop_default()

	warnings.resetwarnings()
	return xy