# -*- coding: iso_8859_1 -*-

import numpy as np
import load, transform

#----------------------------------------------------------------------------------------
#def aperture_photometry(Folder, Imagelist, sizes, xy, xy2, recenter='yes'):
#
#--- purpose ---
#to calculate the relative magnitude of a list of star-like objects with specified positions, on a serie of FITS images, using the method of aperture photometry
#the inputs xy, xy2 must be provided by the function astro.select_stars()
#
#--- input parameters ---
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will load all the files with complete paths = Folder + Imagelist[i]
#sizes = list of 3 elements lists [a,b,c]
#		the 3 values [a,b,c] define the radii of three circles for aperture photometry, with a,b,c being sorted in increasing order
#		example: sizes = [[6,10,20],[7,12,30]]
#xy = list of list of 2 elements lists, giving the [x,y] position of the stars maximum pixel values, as calculated with astro.select_stars()
#		xy[i] gives all object positions [[x1,y1],[x2,y2],...] on the (i+1)th image
#		xy[i][j] gives the [x,y] position of the (j+1)th object on the (i+1)th image
#		xy[i][j][0] gives the x coordinate of the (j+1)th object on the (i+1)th image
#xy2 = list of list of 2 elements lists, giving the [x,y] position of star PSF centroid, as calculated with astro.select_stars()
#		xy[i] gives all object positions [[x1,y1],[x2,y2],...] on the (i+1)th image
#		xy[i][j] gives the [x,y] position of the (j+1)th object on the (i+1)th image
#		xy[i][j][0] gives the x coordinate of the (j+1)th object on the (i+1)th image
#		Remarks:
#		- if xy2 = [] has been set to an empty list by astro.select_stars() then recenter must be set to 'no'
#recenter = string, if equal to 'no' then aperture photometry is performed by centering the measurements disks with the star pixel of max value
#		if equal to 'yes' then the star image is translated with bilinear interpolation, so that its center fits the center of a pixel, 
#		and then aperture photometry is performed by centering the measurement disks on this pixel
#		if equal to 'yes' and the xy coordinates of the PSF centroid have been set to [-1.,-1.] by astro.select_stars() due to a bad PSF fit, then position of star pixel of max value is used instead
#
#--- output parameters ---
#mag_total = this parameter reports all the measured magnitudes
#		mag_total[i][j][k] corresponds to the magnitude of object j (with name: names[j] from astro.select_stars()) on picture i (from file: Imagelist[i]) as measured with method k (with size: sizes[k])
#		in case of error: returned magnitudes are set to 0 if star was to close to image border ; or to 999 if the calculated intensity was negative
#----------------------------------------------------------------------------------------

def aperture_photometry(Folder, Imagelist, sizes, xy, xy2, recenter='yes'):

	print
	print '---------------------------------------------------------------------'
	print '      script for aperture photometry'
	print '---------------------------------------------------------------------'
	
	#loop on images
	mag_total = []
	for i in range(len(Imagelist)):

		#loads relevant image
		print 'Photometry on :',Imagelist[i]
		image = load.getdata(Folder, [Imagelist[i]], text_display='no')

		#loop on stars / asteroids
		mag_stars = []
		for j in range(len(xy[i])):

			#best integer coordinates for object center
			if (recenter == 'yes'):
				if (xy2[i][j][0]!=-1):
					x = np.round(xy2[i][j][0])
					y = np.round(xy2[i][j][1])
					dx = xy2[i][j][0] - x
					dy = xy2[i][j][1] - y
				else:
					x = xy[i][j][0]
					y = xy[i][j][1]
					dx = 0.
					dy = 0.
			else:
				x = xy[i][j][0]
				y = xy[i][j][1]
				dx = 0.
				dy = 0.
							
			#loop on apertures / PSF area sizes for photometry
			mag_methods = []
			for k in range(len(sizes)):		

				if 	(x-sizes[k][0]-1<0)or(x+sizes[k][0]+2>image.shape[0])or(y-sizes[k][0]-1<0)or(y+sizes[k][0]+2>image.shape[1]):
					print "aperture photometry on image", i+1, ": object", j+1, "too close from image border"
					mag_methods.append(0.)
				else:
					#selecting subimage
					xnot = max(0,x-sizes[k][2]-1)
					xrange = min(image.shape[0],x+sizes[k][2]+2)
					ynot = max(0,y-sizes[k][2]-1)
					yrange = min(image.shape[1],y+sizes[k][2]+2)

					subimage = image[xnot:xrange,ynot:yrange]

					#------ re-centering accurately the star ------
					if recenter=='yes':
						if (xy2[i][j][0]!=-1):
							subimage = transform.translate_bilinear(subimage,(-dx,-dy))
					#----------------------------------------------
					
					subimage = subimage[1:-1,1:-1]

					#aperture photometry
					grid = np.indices(subimage.shape)
					radius = (grid[0]-x+xnot+1)**2+(grid[1]-y+ynot+1)**2

					cond1 = radius<=sizes[k][0]**2
					cond2 = (radius>=sizes[k][1]**2)*(radius<=sizes[k][2]**2)

					center = np.extract(cond1,subimage)
					crown = np.extract(cond2,subimage)

					floor = np.ma.median(np.array(crown,dtype=np.float))
					intensity = np.sum(np.array(center,dtype=np.float)-floor)
					if intensity <= 0.:
						print "negative intensity on image", i+1, "for object", j+1
						mag_methods.append(999.)
					else:
						mag_methods.append(-2.5*np.log10(intensity))

			mag_stars.append(mag_methods)
		mag_total.append(mag_stars)

	return mag_total
	