# -*- coding: iso_8859_1 -*-

import time, sys
import numpy as np
import matplotlib.pyplot as plt
import load, displaying, process, process2
import win32api

#----------------------------------------------------------------------------------------
#def select_stars(Folder, Imagelist, threshold, detection_area, images_aligned='no', psf_fit='yes'):
#
#--- purpose ---
#to return the accurate positions of 1 moving object + an arbitrary number of stars on each image in a serie of FITS images
#
#--- input parameters ---
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will load all the files with complete paths = Folder + Imagelist[i]
#threshold = defines the threshold used to detect the stars for image shift calculation
#		always between 0 and 1; reasonnable values are 0.05, 0.10
#		when threshold is set to low values, more stars are considered
#		the actual threshold that is used is np.mean(image) + threshold*(np.max(image)-np.mean(image))
#detection_area = defines the +/- size of the square area used for image shift calculation, local max detection and PSF fitting
#		typical value is detection_area=10, resulting in a square of 21*21 pixels
#images_aligned = string, equal to 'yes' if images are already co-registered in the image list : then no image shift will be calculated
#		otherwise it must be set to 'no', and image shifts are calculated
#psf_fit = string, if equal to 'yes' then the function returns both the location of the star maximum pixel value and the PSF centroid position (PSF model is an elliptical gaussian)
#		otherwise it must be set to 'no', and only location of the star maximum pixel value is reported
#
#--- output parameters ---
#names = list with strings giving names for the identified object = ['moving_object', 'star_1', 'star_2', etc]
#xy = list of list of 2 elements lists, giving the (x,y) position of the stars maximum pixel values
#		xy[i] gives all object positions on the (i+1)th image
#		xy[i][j] gives the (x,y) position of the (j+1)th object on the (i+1)th image
#		xy[i][j][0] gives the x coordinate of the (j+1)th object on the (i+1)th image
#		reported (x,y) coordinates are integers, but are provided as float numbers (e.g. 312.0, 127.0, etc)
#xy2 = list of list of 2 elements lists, giving the (x,y) position of star PSF centroid
#		xy[i] gives all object positions on the (i+1)th image
#		xy[i][j] gives the (x,y) position of the (j+1)th object on the (i+1)th image
#		xy[i][j][0] gives the x coordinate of the (j+1)th object on the (i+1)th image
#		Remarks:
#		- if psf_fit = 'no' then xy2 = [] is an empty list
#		- each time the psf fit gives a centroid that is too far away from the pixel with max signal, a warning message is displayed 
#		in the console window and (-1.,-1.) is stored instead of the (x,y) PSF centroid coordinates
#----------------------------------------------------------------------------------------

def select_stars(Folder, Imagelist, threshold, detection_area, images_aligned='no', psf_fit='yes'):

	print
	print '---------------------------------------------------------------------'
	print '      script for star selection'
	print '---------------------------------------------------------------------'
	
	#------------------------------------------------------------------------------------
	#	loading basic information
	#------------------------------------------------------------------------------------

	#--- initialising main variables ---
	time_format = "%Y-%m-%dT%H:%M:%S"
	x=[]		#X-Y positions on 1st image
	y=[]
	dx=[]		#X-Y positions change per unit time
	dy=[]
	names = []
	excl = []

	#------ first image ------
	image1 = load.getdata(Folder, [Imagelist[0]])
	date1 = load.getheadervalues(Folder, [Imagelist[0]], 'DATE-OBS')
	t1 = time.mktime(time.strptime(date1[0].split('.')[0],time_format))
	
	#------ last image ------
	image2 = load.getdata(Folder, [Imagelist[-1]])
	date2 = load.getheadervalues(Folder, [Imagelist[-1]], 'DATE-OBS')
	t2 = time.mktime(time.strptime(date2[0].split('.')[0],time_format))

	#------ setting up console windows ------
	win32api.maximize_window(u"C:\\WINDOWS\\system32\\cmd.exe")
	sx, sy = win32api.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
	
	#------------------------------------------------------------------------------------
	#	selecting one moving object
	#------------------------------------------------------------------------------------

	fig = displaying.display(image1, title="First image")
	fx, fy = win32api.get_window_position("First image")
	win32api.move_window("First image", [sx, fy])
	win32api.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	print
	print 'How many moving objects is there on the pictures ?'
	nb = np.int(raw_input('nb of moving objects = '))

	if nb>0:
		print
		print 'First image : right-click on exact position of moving objects'
		win32api.set_focus("First image")
		xy1 = displaying.select(fig,nb)

		fig2 = displaying.display(image2, title="Last image")
		fx, fy = win32api.get_window_position("Last image")
		win32api.move_window("Last image", [sx, fy])
		print
		print 'Last image : right-click on exact position of moving objects (same order)'
		xy2 = displaying.select(fig2,nb)
		
		plt.close()
		
		for object in xy1:
			excl.append([object[0]-10, object[0]+10, object[1]-10, object[1]+10])

		#------ shift between both images ------
		if images_aligned == 'no':
			xs, ys = process2.find_shift_stars_matching(image1, image2, threshold, detection_area, excluded = excl)
		else:
			xs = 0.
			ys = 0.

		for i in range(nb):
			x1 = xy1[i][0]
			y1 = xy1[i][1]

			if 	(x1-detection_area<0)or(x1+detection_area+1>image1.shape[0])or(y1-detection_area<0)or(y1+detection_area+1>image1.shape[1]):
				print 'object',i+1,'is too close from image border'
				print 'x1 and y1 must be >= ',detection_area
				print 'x1 must be <= ',image1.shape[0]-detection_area-1
				print 'y1 must be <= ',image1.shape[1]-detection_area-1
				raw_input('script is terminated: press ENTER to close window')
				sys.exit()
			else:
				subimage1 = image1[x1-detection_area:x1+detection_area+1, y1-detection_area:y1+detection_area+1]
				xcenter,ycenter = np.where(subimage1==np.max(np.max(subimage1)))
				max1 = [np.float(xcenter[0]) + x1 - detection_area, np.float(ycenter[0]) + y1 - detection_area]

			x2 = xy2[i][0]
			y2 = xy2[i][1]
	
			if 	(x2-detection_area<0)or(x2+detection_area+1>image2.shape[0])or(y2-detection_area<0)or(y2+detection_area+1>image2.shape[1]):
				print 'object is too close from image border'
				print 'x2 and y2 must be >= ',detection_area
				print 'x2 must be <= ',image2.shape[0]-detection_area-1
				print 'y2 must be <= ',image2.shape[1]-detection_area-1
				raw_input('script is terminated: press ENTER to close window')
				sys.exit()
			else:
				subimage2 = image2[x2-detection_area:x2+detection_area+1, y2-detection_area:y2+detection_area+1]
				xcenter,ycenter = np.where(subimage2==np.max(np.max(subimage2)))
				max2 = [np.float(xcenter[0]) + x2 - detection_area, np.float(ycenter[0]) + y2 - detection_area]
			
			#--- collecting information ---
			x.append(max1[0])
			y.append(max1[1])
			dx.append((max2[0]-max1[0]-xs)/(t2-t1))
			dy.append((max2[1]-max1[1]-ys)/(t2-t1))
			names.append('moving_object_'+str(i+1))
		
	#------------------------------------------------------------------------------------
	#	selecting stars
	#------------------------------------------------------------------------------------

	print
	win32api.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	print 'How many stars do you want to analyse ?'
	nb = np.int(raw_input('nb of stars = '))
	
	if nb>0:
		print
		print 'First image : right-click on exact positions of',nb,'stars'
		win32api.set_focus("First image")
		xy1 = displaying.select(fig,nb)
		
		for i in range(nb):
			x1 = xy1[i][0]
			y1 = xy1[i][1]

			if 	(x1-detection_area<0)or(x1+detection_area+1>image1.shape[0])or(y1-detection_area<0)or(y1+detection_area+1>image1.shape[1]):
				print 'object',i+1,'is too close from image border'
				print 'x1 and y1 must be >= ',detection_area
				print 'x1 must be <= ',image1.shape[0]-detection_area-1
				print 'y1 must be <= ',image1.shape[1]-detection_area-1
				raw_input('script is terminated: press ENTER to close window')
				sys.exit()
			else:
				subimage1 = image1[x1-detection_area:x1+detection_area+1, y1-detection_area:y1+detection_area+1]
				xcenter,ycenter = np.where(subimage1==np.max(np.max(subimage1)))
				max1 = [np.float(xcenter[0]) + x1 - detection_area, np.float(ycenter[0]) + y1 - detection_area]

			x.append(max1[0])
			y.append(max1[1])
			dx.append(0.)
			dy.append(0.)
			names.append('star_'+str(i+1))
	
	plt.close()
	
	#------------------------------------------------------------------------------------
	#	check that something has been selected
	#------------------------------------------------------------------------------------

	if len(x)==0:
		print
		raw_input('no object has been selected, press ENTER to close window')
		sys.exit()

	#------------------------------------------------------------------------------------
	#	calculating positions of local maxima on all frames
	#------------------------------------------------------------------------------------

	print
	print '---------------------------------------------------------------------'

	xy = []
	for i in range(len(Imagelist)):					#loop on images
	
		print 'calculating local maxima on :',Imagelist[i]
		if i==0:
			xy_image = [[x[j],y[j]] for j in range(len(x))]
		else:
			image = load.getdata(Folder, [Imagelist[i]], text_display='no')
			date = load.getheadervalues(Folder, [Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			if images_aligned == 'no':
				xs, ys = process2.find_shift_stars_matching(image1, image, threshold, detection_area, excluded=excl)
			else:
				xs = 0.
				ys = 0.
			
			xy_image = []
			for j in range(len(x)):						#loop on stars
				xi = np.round(x[j]+xs+(t-t1)*dx[j])
				yi = np.round(y[j]+ys+(t-t1)*dy[j])
				if 	(xi-detection_area<0)or(xi+detection_area+1>image.shape[0])or(yi-detection_area<0)or(yi+detection_area+1>image.shape[1]):
					print 'object',j+1,'is too close from image',i+1,'border'
					xy_image.append([-1., -1.])
				else:
					subimage = image[xi-detection_area:xi+detection_area+1, yi-detection_area:yi+detection_area+1]
					xcenter,ycenter = np.where(subimage==np.max(np.max(subimage)))
					maxi = [np.float(xcenter[0]) + xi - detection_area, np.float(ycenter[0]) + yi - detection_area]
					xy_image.append(maxi)

		xy.append(xy_image)

	#------------------------------------------------------------------------------------
	#	psf fitting
	#------------------------------------------------------------------------------------
	
	xy2 = []
	
	if psf_fit=='yes':	
		for i in range(len(Imagelist)):					#loop on images
		
			print 'PSF fitting on :',Imagelist[i]
			if i==0:
				image = image1
			else:
				image = load.getdata(Folder, [Imagelist[i]], text_display='no')

			xy_image = []
			for j in range(len(x)):						#loop on stars
				
				xi = xy[i][j][0]
				yi = xy[i][j][1]

				if 	(xi-detection_area<0)or(xi+detection_area+1>image.shape[0])or(yi-detection_area<0)or(yi+detection_area+1>image.shape[1]):
					print 'object',j+1,'is too close from image',i+1,'border'
					xy_image.append([-1., -1.])
				else:
					p = process.fit_gauss_elliptical([xi-detection_area,yi-detection_area],image[xi-detection_area:xi+detection_area+1, yi-detection_area:yi+detection_area+1])				
					maximum, floor, height, mean_x, mean_y, fwhm_1, fwhm_2, angle = p
					if np.abs(xi-mean_x)+np.abs(yi-mean_y)>=2:
						print 'bad psf fit for object',j+1,'on image',i+1
						xy_image.append([-1., -1.])
					else:
						xy_image.append([mean_x, mean_y])
				
			xy2.append(xy_image)

	return names, xy, xy2
