# -*- coding: iso_8859_1 -*-

import numpy as np
import sys
import process2, stats, transform

#----------------------------------------------------------------------------------------
#def find_shift_by_triangles(image1, image2, threshold, detection_area, excluded=[], included=[], text_display='no'):
#
#--- purpose ---
#calculates the non-integer x,y shift between two images of the same star field
#algorithm is based on the method of triangles (as described in http://ned.ipac.caltech.edu/level5/Stetson/Stetson5_2.html)
#
#--- input parameters ---
#image1 = 2D numpy array containing data of first image
#image2 = 2D numpy array containing data of second image
#		image1[j][k] or image2[j][k] contain the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#threshold = defines the threshold used to detect the stars
#		always between 0 and 1; reasonnable values are 0.05, 0.10
#		when threshold is set to low values, more stars are considered
#		the actual threshold that is used is np.mean(image) + threshold*(np.max(image)-np.mean(image))
#detection_area = defines the +/- size of the square area used for star detection with astro.find_stars()
#		typical value is detection_area=10, resulting in a square of 21*21 pixels
#excluded = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image that are excluded from calculations (e.g. galaxie(s), artefact(s), moving object(s)..)
#included = [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]] defines a list of rectangular zones [xi1,xi2,yi1,yi2] on first image on which all calculations must be done 
#		(all parts of the initial image not belonging to at least one of these zones are rejected)
#text_display = string, if equal to 'yes' then a matrix of stars matching between the 2 images is displayed, to check result quality
#		otherwise (default mode), nothing is displayed
#
#--- output parameters ---
#result = list of [dx,dy] 2 elements giving the calculated shifts respectively along the x (vertical) and y (horizontal) axes
#----------------------------------------------------------------------------------------

def find_shift_by_triangles(image1, image2, threshold, detection_area, excluded=[], included=[], text_display='no'):

	step = 2.			#shift difference value below which shifts are assumed to belong to the same translation scenario
	#-------------------------------------------------------------------------------------
	#	finding stars
	#-------------------------------------------------------------------------------------

	stars1 = process2.find_stars(image1, threshold, detection_area, excluded, included)
	stars2 = process2.find_stars(image2, threshold, detection_area, excluded, included)

	nb1 = stars1.shape[0]
	nb2 = stars2.shape[0]
	
	if (nb1==0)or(nb2==0):
		raw_input('no stars could be detected, shift will not be calculated.\nPress Enter to close')
		sys.exit()		

	#-------------------------------------------------------------------------------------
	#	looking for triangles
	#-------------------------------------------------------------------------------------

	tri_1 = []
	for i in range(nb1):
		for j in range(i+1,nb1):
			for k in range(j+1,nb1):
				side_1 = np.sqrt((stars1[i,0]-stars1[j,0])**2.)
				side_2 = np.sqrt((stars1[i,0]-stars1[k,0])**2.)
				side_3 = np.sqrt((stars1[j,0]-stars1[k,0])**2.)
				list = [side_1, side_2, side_3]
				
				a = max(list)
				list.remove(a)
				b = max(list)
				c = min(list)

				ratio_1 = b/a
				ratio_2 = c/a

				if (a==side_1)and(b==side_2):
					tri_1.append([ratio_1, ratio_2, k, j, i])
				elif (a==side_1)and(b==side_3):
					tri_1.append([ratio_1, ratio_2, k, i, j])
				elif (a==side_2)and(b==side_1):
					tri_1.append([ratio_1, ratio_2, j, k, i])
				elif (a==side_2)and(b==side_3):
					tri_1.append([ratio_1, ratio_2, j, i, k])
				elif (a==side_3)and(b==side_1):
					tri_1.append([ratio_1, ratio_2, i, k, j])
				elif (a==side_3)and(b==side_2):
					tri_1.append([ratio_1, ratio_2, i, j, k])
				
	tri_2 = []
	for i in range(nb2):
		for j in range(i+1,nb2):
			for k in range(j+1,nb2):
				side_1 = np.sqrt((stars2[i,0]-stars2[j,0])**2.)
				side_2 = np.sqrt((stars2[i,0]-stars2[k,0])**2.)
				side_3 = np.sqrt((stars2[j,0]-stars2[k,0])**2.)
				list = [side_1, side_2, side_3]
				
				a = max(list)
				list.remove(a)
				b = max(list)
				c = min(list)
				
				ratio_1 = b/a
				ratio_2 = c/a

				if (a==side_1)and(b==side_2):
					tri_2.append([ratio_1, ratio_2, k, j, i])
				elif (a==side_1)and(b==side_3):
					tri_2.append([ratio_1, ratio_2, k, i, j])
				elif (a==side_2)and(b==side_1):
					tri_2.append([ratio_1, ratio_2, j, k, i])
				elif (a==side_2)and(b==side_3):
					tri_2.append([ratio_1, ratio_2, j, i, k])
				elif (a==side_3)and(b==side_1):
					tri_2.append([ratio_1, ratio_2, i, k, j])
				elif (a==side_3)and(b==side_2):
					tri_2.append([ratio_1, ratio_2, i, j, k])

	#-------------------------------------------------------------------------------------
	#	comparing triangles
	#-------------------------------------------------------------------------------------
	threshold = 0.01
	
	match = np.zeros((nb1, nb2))
	for i in range(len(tri_1)):
		for j in range(len(tri_2)):
			if (np.abs(tri_1[i][0]-tri_2[j][0]) + np.abs(tri_1[i][1]-tri_2[j][1])) < threshold:
				match[tri_1[i][2],tri_2[j][2]] += 1.
				match[tri_1[i][3],tri_2[j][3]] += 1.
				match[tri_1[i][4],tri_2[j][4]] += 1.

	if text_display=='verbose':
		print 'star matching matrix ='
		for i in range(nb1):
			print match[i,:]

	xy = []
	maxmax = np.max(match.flatten())
	for i in range(nb1):
		if np.max(match[i,:])>maxmax/2.:
			j = np.argmax(match[i,:])
			xy.append([stars1[i,0], stars1[i,1], stars2[j,0], stars2[j,1]])

	#-------------------------------------------------------------------------------------
	#	calculating shifts
	#-------------------------------------------------------------------------------------
	dxy = []
	for coord in xy:
		dxy.append([coord[2]-coord[0], coord[3]-coord[1]])

	return [np.mean(np.array(dxy)[:,0]), np.mean(np.array(dxy)[:,1])]
	