# -*- coding: iso_8859_1 -*-

import pyfits
import os
import numpy as np

#----------------------------------------------------------------------------------------
#def save_fits(images, headers, Folder, Imagelist):
#
#--- purpose ---
#to save a list of FITS images, given their data and header information
#
#--- input parameters ---
#images = 3D numpy array containing data for the list of images. All images must have the same size
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		remark: the 3D array can contain only a single image, but use of 2D array is then recommended
#		OR
#		= 2D numpy array containing data of a single image
#		images[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#headers = header information for the image list, in the form of a list (over all images) of lists (over all headers) of 3-element lists
#		headers[i] contains all the FITS headers of the (i+1)th image
#		headers[i][j] contains the full record of the (j+1)th header of the (i+1)th image
#		headers[i][j][0] contains the FITS keyword of the (j+1)th header of the (i+1)th image
#		headers[i][j][1] contains the value of the (j+1)th header of the (i+1)th image
#		headers[i][j][2] contains the comment of the (j+1)th header of the (i+1)th image
#Folder = string, gives the folder where the FITS files are stored
#		if Folder contains non-ASCII characters (such as , , , etc) it must specified as a unicode string, with the prefix "u"
#		examples: "C:\\Images\\", u"C:\\Astroides\\"
#Imagelist = list of strings, each string is the name of one FITS file
#		the function will load all the files with complete paths = Folder + Imagelist[i]
#
#--- output parameters ---
#no outputs
#----------------------------------------------------------------------------------------

def save_fits(images, headers, Folder, Imagelist):

	#--- conditioning data ---
	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))	
	images = np.array(np.round(images,0),dtype=np.int16)

	for i in range(images.shape[0]):
		#--- create FITS ---
		hdu = pyfits.PrimaryHDU()
		hdu.data = images[i,:,:]
		del hdu.header['EXTEND']
		for hh in headers[i]:
			hdu.header.update(hh[0],hh[1],hh[2])
		
		#--- save FITS ---
		if os.path.exists(Folder+Imagelist[i]):
			os.remove(Folder+Imagelist[i])
		hdu.writeto(Folder+Imagelist[i])
	
	return 0