# -*- coding: iso_8859_1 -*-

import numpy as np
import process, process2
from ctypes import *

#----------------------------------------------------------------------------------------
#def get_median(images):
#
#--- purpose ---
#calculates the median values of all pixels on each image in a list of images
#if the image data are integer, they are converted to float before computation in order to get the half integer values correctly
#the algorithm to calculate the median values is based on the Wirth method
#a C implementation from Devillard (from http://ndevilla.free.fr/median/median/index.html) is used, after correction of a typo
#the original code is as a DLL, the DLL is called from Python with the module c_types
#
#--- input parameters ---
#images = 3D numpy array containing data for list of images. All images must have the same size
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		remark: the 3D array can contain only a single image, but use of 2D array is then recommended
#		OR
#		= 2D numpy array containing data of a single image
#		images[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#
#--- output parameters ---
#result = python list containing the median values (float values)
#----------------------------------------------------------------------------------------

def get_median(images):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim) 
	z=imag_out()

	cdll.median.Median2(dim,dim_x,dim_y,byref(y),byref(z))

	result = []
	result[:] = z[:]
	return result
	
#----------------------------------------------------------------------------------------
#def mean_psf(image, threshold, detection_area, psf_fit_area):
#
#--- purpose ---
#calculates the median parameters of the PSF of the brightest stars on one image
#the function is using the 2D elliptical Gaussian PSF model (similar to the Gaussian PSF used in PRISM software)
#
#--- input parameters ---
#image = 2D numpy array containing data for the image to be analysed.
#		image[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#threshold = defines the threshold used to detect the stars on which PSF will be fitted
#		always between 0 and 1; reasonnable values are 0.05, 0.10
#		when threshold is set to low values, more stars are considered and calculations are longer
#		the actual level that is used is np.mean(image) + threshold*(np.max(image)-np.mean(image))
#detection_area = defines the +/- size of the square area used for star detection with astro.find_stars()
#		typical value is detection_area=10, resulting in a square of 21*21 pixels
#psf_fit_area = defines the +/- size of the square area used for PSF fitting
#		typical value is 10, resulting in a square of 21*21 pixels
#
#--- output parameters ---
#result = list with 3 values = [median smallest fwhm, median largest fwhm, median azimuth of largest fwhm measured from vertical axis clockwise]
#----------------------------------------------------------------------------------------

def mean_psf(image, threshold, detection_area, psf_fit_area):

	#fitting stars
	stars = process2.find_stars(image, threshold, detection_area)
	fwhm_1 = []
	fwhm_2 = []
	cosine = []
	sine = []
	
	for xy in stars:
		#print xy
		p = process.fit_gauss_elliptical([xy[0]-psf_fit_area,xy[1]-psf_fit_area],image[xy[0]-psf_fit_area:xy[0]+psf_fit_area+1, xy[1]-psf_fit_area:xy[1]+psf_fit_area+1])
		maxi, floor, height, mean_x, mean_y, fwhm_small, fwhm_large, angle = p
				
		fwhm_1.append(fwhm_small)
		fwhm_2.append(fwhm_large)
		cosine.append(np.cos(angle))
		sine.append(np.sin(angle))
		
	return [np.ma.mean(fwhm_1), np.ma.mean(fwhm_2), np.arctan2(np.ma.mean(sine),np.ma.mean(cosine))]