# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *

#----------------------------------------------------------------------------------------
#def translate_bilinear(images, shift):
#def rotate_bilinear(images, xy_axis, angle):
#def scale_bilinear(images, xy_center, scaling):
#def translate_Bspline(images, shift, spline_degree):
#def rotate_Bspline(images, xy_axis, angle, spline_degree):
#def scale_Bspline(images, xy_center, scaling, spline_degree):
#
#--- purpose ---
#these functions apply a non-integer translation, a rotation or scaling to one or a list of input images
#B-spline interpolation is re-using existing C code from http://bigwww.epfl.ch/thevenaz/interpolation/
#
#--- input parameters ---
#images = 3D numpy array containing data for list of images. All images must have the same size
#		images[i][j][k] contains the pixel of the ith image, on jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#		remark: the 3D array can contain only a single image, but use of 2D array is then recommended
#		OR
#		= 2D numpy array containing data of a single image
#		images[j][k] contains the pixel of the jth row (j changes in vertical direction) and kth column (k changes in horizontal direction)
#shift (only translation functions) = two-elements list [x,y] giving the shift vector to be applied to the images in the list 
#angle (only rotation functions) = angle of rotation (measured clockwise)
#xy_axis (only rotation functions) = two-elements list [x,y] giving the (x,y) coordinates of the rotation axis
#scaling (only scaling functions) = scaling factor to be applied
#xy_center (only scaling functions) = two-elements list [x,y] giving the (x,y) coordinates of the pixel stays at the same position after scaling
#spline_degree (only B-spline interpolation for the geometric operations) = degree of B-spline, can be = 2,3,4,5,6,7,8 or 9
#
#--- output parameters ---
#result = 3D (or 2D, depending on input) numpy array containing data of translated image(s)
#----------------------------------------------------------------------------------------

def translate_Bspline(images, shift, spline_degree):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.Bspline.Translate(dim,dim_x,dim_y,byref(y),byref(dxy),spline_degree,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def rotate_Bspline(images, xy_axis, angle, spline_degree):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.Bspline.Rotate(dim,dim_x,dim_y,byref(y),byref(xy_dll),angle_dll,spline_degree,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def scale_Bspline(images, xy_center, scaling, spline_degree):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]

	scaling_dll = c_double(scaling)
	
	cdll.Bspline.Scale(dim,dim_x,dim_y,byref(y),byref(xy_dll),scaling_dll,spline_degree,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def translate_bilinear(images, shift):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.bilinear.Translate(dim,dim_x,dim_y,byref(y),byref(dxy),byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def rotate_bilinear(images, xy_axis, angle):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.bilinear.Rotate(dim,dim_x,dim_y,byref(y),byref(xy_dll),angle_dll,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))

def scale_bilinear(images, xy_center, scaling):

	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))
		
	dim, dim_x, dim_y = images.shape

	imag_in = c_double * (dim*dim_x*dim_y) 
	y=imag_in()
	y[:] = images.flatten()[:]

	imag_out = c_double * (dim*dim_x*dim_y) 
	z=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]
	
	scaling_dll = c_double(scaling)
	
	cdll.bilinear.Scale(dim,dim_x,dim_y,byref(y),byref(xy_dll),scaling_dll,byref(z))

	result = []
	result[:] = z[:]
	return np.reshape(result,(dim_x,dim_y))
	