# -*- coding: iso_8859_1 -*-

import astro
import os, datetime

#------------------------------------------------------------------------------------
#	input parameters
#------------------------------------------------------------------------------------

#--- images ---
Folder = u"C:\\Documents and Settings\\Jrme\\Bureau\\21apr2011\\processing\\Hamburga\\"
#Folder = u"C:\\Documents and Settings\\jerome caron\\Desktop\\Perso astro\\06mar2011\\processing python\\Chryseis\\"

#Folder = u"C:\\ASTROPHOTOS\\Audine_images\\2011_04\\12apr2011\\processing\\Hamburga\\"
Imagelist = ["Hamburga-calibrated-" + str(i+1) + ".fit" for i in range(177)] + ["Hamburga-calibrated-" + str(i+1) + ".fit" for i in range(181,230)]


#--- output file ---
outputfile = "magnitudes.txt"

#parameters to find image shifts
threshold = 0.1		#for star detection
detection_area = 5		#for PSF fitting

#radii for aperture photometry (pixels)
#sizes = [[4.5,8,18],[4.5,8,22],[5,10,20],[5,10,24],[5.5,10,20],[5.5,10,24],[6,10,20],[6,10,24],[6.5,10,20],[6.5,10,24],[7,12,24],[7,12,28]]
#sizes = [[2.5,8,18],[2.5,8,22],[3,8,18],[3,8,22],[3.5,8,18],[3.5,8,22],[4,8,18],[4,8,22],[7.5,14,26],[7.5,14,32],[8,16,26],[8,16,32],[8.5,16,30],[8.5,16,36],[9,16,30],[9,16,36],[9.5,16,30],[9.5,16,36]]
#sizes = [[2.5,8,18],[3,8,18],[3.5,8,18],[4,8,18],[4.5,8,18],[5,10,20],[5.5,10,20],[6,10,20],[6.5,10,20],[7,12,24],[7.5,14,26],[8,16,26],[8.5,16,30],[9,16,30],[9.5,16,30]]
sizes = [[2,8,40],[2.5,8,40],[3,8,42],[3.5,8,42],[4,8,42],[4.5,8,42],[5,10,44],[5.5,10,44],[6,10,44]]
#sizes = [[5,10,24]]

#--- choice of photometry method ---
images_aligned_flag = 'no'
recenter_flag = 'yes'

#------------------------------------------------------------------------------------
#	calling photometry function
#------------------------------------------------------------------------------------

#------ identification of target and reference stars ------
names, xy, xy2 = astro.select_stars(Folder, Imagelist, threshold, detection_area, images_aligned=images_aligned_flag, psf_fit=recenter_flag)

#------ aperture photometry ------
mag_total = astro.aperture_photometry(Folder, Imagelist, sizes, xy, xy2, recenter=recenter_flag)

#------ date information ------
dateobs = astro.julian_day(astro.getheadervalues(Folder, Imagelist, 'DATE-OBS'))

#------ coordinates that have been used for photometry ------
if recenter_flag=='yes':
	xy3 =[[[xy2[i][j][0],xy2[i][j][1]] if (xy2[i][j][0]!=-1) else [xy[i][j][0],xy[i][j][1]] for j in range(len(xy[0]))] for i in range(len(xy))]
else:
	xy3 = xy

#------------------------------------------------------------------------------------
#	saving data
#------------------------------------------------------------------------------------

if os.path.exists(Folder+outputfile):
	os.remove(Folder+outputfile)

f = open(Folder+outputfile, 'w')

#--- file header ---
f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
f.write(str(len(Imagelist))+' images processed\n')
f.write('first image = '+(Folder).encode('iso_8859_1')+Imagelist[0]+'\n\n')

#--- stars X,Y coordinates ---
f.write('object coordinates\n')
for j in range(len(xy3[0])):
	f.write('X '+names[j]+'\t'+'Y '+names[j]+'\t')
f.write('\n')
for i in range(len(Imagelist)):
	for j in range(len(xy3[i])):
		f.write(str(xy3[i][j][0])+'\t'+str(xy3[i][j][1])+'\t')
	f.write('\n')
f.write('\n')

#--- measures magnitudes ---
for k in range(len(sizes)):
	if len(sizes[k])==1:
		f.write('magnitudes from PSF fitting over area +/- '+str(sizes[k][0])+' pixels wide\n')
	elif len(sizes[k])==3:
		f.write('magnitudes from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

	f.write('Julian day\t')
	for j in range(len(xy3[i])):
		f.write(names[j]+'\t')
	f.write('\n')
	for i in range(len(Imagelist)):
		f.write(str(dateobs[i])+'\t')
		for j in range(len(xy3[i])):
			f.write(str(mag_total[i][j][k])+'\t')
		f.write('\n')
	f.write('\n')

f.close()

raw_input("ok")