# -*- coding: iso_8859_1 -*-

import astro
import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------------------------------------------
#	information on source FITS data
#------------------------------------------------------------------------------------

Folder = u"C:\\Documents and Settings\\Jrme\\Bureau\\29mar2011\\processing\\Chryseis\\"

#--- input frames ---
Image = "Chryseis-calibrated-"
Imagelist = [Image + str(i+1) + ".fit" for i in range(304)]

#--- output frames ---
Saving = "Chryseis-aligned-bypython-"
Savinglist = [Saving + str(i+1) + ".fit" for i in range(304)]

#--- additional information (for FITS headers) ---
Tel = 'visac_short_f'
Obj = 'Chryseis - calibrated and aligned'

#parameters to find image shifts
threshold = 0.05		#for star detection
psf_fit_area = 12		#for PSF fitting

#------------------------------------------------------------------------------------
#	reference image
#------------------------------------------------------------------------------------

Exp = astro.getheadervalues(Folder, Imagelist, 'EXPOSURE')
Dateobs = astro.getheadervalues(Folder, Imagelist, 'DATE-OBS')

#--- Processing and saving ---
image0 = astro.getdata(Folder, [Imagelist[0]])
headers0 = astro.getheaders(Folder, [Imagelist[0]])
astro.save_fits(image0, headers0, Folder, [Savinglist[0]])
print Savinglist[0] + ' saved'

#--- Choosing zone to exclude for shift calculation ---
astro.display(image0)

print
print 'Specify boundaries of zone to be excluded to calculate shift'
print 'if moving object (asteroid), only exclude a few pixels giving its position on 1st image'
xmin = np.float32(raw_input('x boundaries (vertical): xmin = '))
xmax = np.float32(raw_input('x boundaries (vertical): xmax = '))
ymin = np.float32(raw_input('y boundaries (horizontal): ymin = '))
ymax = np.float32(raw_input('y boundaries (horizontal): ymax = '))

plt.close()

#------------------------------------------------------------------------------------
#	images calibration, co-registration and saving
#------------------------------------------------------------------------------------

for i in range(1,len(Imagelist)):
	#loading
	imagei = astro.getdata(Folder, [Imagelist[i]])

	#calculate shifts
	dx,dy = astro.find_shift_stars_matching(image0, imagei, threshold, psf_fit_area, excluded = [[xmin, xmax, ymin, ymax]],text_display='yes')
	print 'shifts (vert, hor) wrt reference image:', dx, dy

	#shift image
	imagei_aligned = astro.translate_bilinear(imagei, (-dx,-dy))

	#save image
	headersi = astro.getheaders(Folder, [Imagelist[i]])
	astro.save_fits(imagei_aligned, headersi, Folder, [Savinglist[i]])
	print Savinglist[i] + ' saved'

raw_input("Press ENTER to close")

